#include <cmath>
#include <cstdlib>
#include <iostream>
#include <fstream>
#include <string>
#include <sstream>

#include "functions.hpp"

using namespace std;

int main()
{
    int N=20; //Pontok száma dimenziónként

    double R=1; //A gömb sugara
    double a=1; //A gömb hődiffúziós tényezője
    double b=0.0000001; //A sugárzási hőveszteség együtthatója
    double omega=5; //A forgó hőforrás körfrekvenciája

    double *theta=(double*)malloc(N*sizeof(double)); //Az egyes indexekhez tartozó szögek tömbjei
    double *phi=(double*)malloc(N*sizeof(double));

    double *temp=(double*)malloc(N*sizeof(double)); //Ideiglenes tömb a fűtés számításához

    linspace(theta,N,-M_PI_2,M_PI_2);   //A szögtömbök feltöltése
    linspace(phi,N+1,-M_PI,M_PI);

    double *u=(double*)malloc(N*N*sizeof(double));  //A hőmérséklet és annak laplaszát tartalmazó tömbök
    double *lu=(double*)malloc(N*N*sizeof(double));

    double t0=0;    //Kezdő időpont
    double tmax=2; //Végpont
    double dt=0.00001;  //Lépésköz
    double t=t0;    //A futó időváltozó

    int tN=0;   //Az időlépések számlálója

    int kskp=0; //Az adatrögzítés lépésközszámlálója
    int Nskp=200; //Az adatrögzítés lépésköze

    init(u,100,N);  //A hőmérséklet inicializálása konstans hőmérsékleten

    ofstream results("results.txt");    //Adatok rögzítéséhez szükséges fájl nyitása

    for(int k=0; k<N; k++) //A kezdőpillanat rögzítése
    {
        for(int l=0;l<N;l++)
        {
            results << t << '\t' << theta[k] << '\t' << phi[l] << '\t' << u[matin(k,l,N)] << '\n';
        }
        results << t << '\t' << theta[k] << '\t' << phi[N+1] << '\t' << u[matin(k,0,N)] << '\n';
        results << endl;
    }

    double heat; //A fűtés változója

    while(t<tmax)   //A szimuláció futtatása
    {
        t=t+dt;
        kskp++;

        lapl(lu, u, theta, R, N);

        for(int k=0; k<N; k++)
        {
            for(int l=0;l<N;l++)
            {
                heat=3*calcheat(omega,t,k,l,N,theta,phi,temp);
                u[matin(k,l,N)]=u[matin(k,l,N)]+dt*(a*lu[matin(k,l,N)]-b*pow(u[matin(k,l,N)],4)+heat);
                if(kskp==Nskp)
                {
                    results << t << '\t' << theta[k] << '\t' << phi[l] << '\t' << u[matin(k,l,N)] << '\n';
                }

            }
            if(kskp==Nskp)
            {
                results << t << '\t' << theta[k] << '\t' << phi[N+1] << '\t' << u[matin(k,0,N)] << '\n';
                results << endl;
            }
        }
        if(kskp==Nskp)
        {
            cout << t << endl;
            tN++;
            kskp=0;
        }
    }

    results.close();

    ofstream plotFile("plotFile.plt"); //A gnuplotot inicializáló fájl írása

    plotFile << "R=" << R << "; dt=" << dt << "; tN=" << tN << "; N=" << N << endl;
    plotFile << "set xrange[-3.5:3.5]; set yrange[-2:2]; set autoscale z; set autoscale cb; set ticslevel 0; set grid; set xlabel \"Phi\"; set ylabel \"Theta\" rotate by 0" << endl;
    plotFile << "set xtics format \"%.2Ppi\" autofreq pi/4; set ytics format \"%.2P{/Symbol p}\" autofreq pi/4;" << endl;
    plotFile << "set term pngcairo size 800,450 enh truecolor; set pm3d depthorder interpolate 4,4; set view map;" << endl;
    plotFile << "do for [k=0:tN] {  set output \"anim/results_\".k.\".png\"; set title \"Frame #\".k; splot \"results.txt\" every 1:1::0+N*k::N-1+N*k u 3:2:(0):4 w pm3d title \""  }" << endl;
    plotFile << "set output";

    plotFile.close();

    system("gnuplot plotFile.plt"); //A gnuplot futtatása

    return 0;
}
